@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2016 by Farsight Security, Inc.
+ * Copyright (c) 2013, 2014, 2016-2017 by Farsight Security, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -239,7 +239,7 @@ fstrm_iothr_init(const struct fstrm_iothr_options *opt,
 	/*
 	 * Some platforms have a ridiculously low IOV_MAX, literally the lowest
 	 * value even allowed by POSIX, which is lower than our conservative
-	 * FSTRM_IOTHR_OUTPUT_QUEUE_SIZE_DEFAULT. Accomodate these platforms by
+	 * FSTRM_IOTHR_OUTPUT_QUEUE_SIZE_DEFAULT. Accommodate these platforms by
 	 * silently clamping output_queue_size to IOV_MAX.
 	 */
 	if (iothr->opt.output_queue_size > IOV_MAX)
@@ -294,7 +294,7 @@ fstrm_iothr_init(const struct fstrm_iothr_options *opt,
 	res = pthread_condattr_init(&ca);
 	assert(res == 0);
 
-#if HAVE_CLOCK_GETTIME
+#if HAVE_CLOCK_GETTIME && HAVE_PTHREAD_CONDATTR_SETCLOCK
 	res = pthread_condattr_setclock(&ca, iothr->clkid_pthread);
 	assert(res == 0);
 #endif
@@ -361,6 +361,9 @@ fstrm_iothr_destroy(struct fstrm_iothr **iothr)
 		(*iothr)->shutting_down = true;
 		pthread_cond_signal(&(*iothr)->cv);
 		pthread_join((*iothr)->thr, NULL);
+		pthread_cond_destroy(&(*iothr)->cv);
+		pthread_mutex_destroy(&(*iothr)->cv_lock);
+		pthread_mutex_destroy(&(*iothr)->get_queue_lock);
 
 		/* Destroy the writer by calling its 'destroy' method. */
 		(void)fstrm_writer_destroy(&(*iothr)->writer);
@@ -609,7 +612,11 @@ fstrm__iothr_thr(void *arg)
 
 		struct timespec ts;
 #if HAVE_CLOCK_GETTIME
+#if HAVE_PTHREAD_CONDATTR_SETCLOCK
 		int rv = clock_gettime(iothr->clkid_pthread, &ts);
+#else
+		int rv = clock_gettime(CLOCK_REALTIME, &ts);
+#endif
 		assert(rv == 0);
 #else
 		my_gettime(-1, &ts);
