@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2016 by Farsight Security, Inc.
+ * Copyright (c) 2013-2014, 2016-2017 by Farsight Security, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 
 #if HAVE_CLOCK_GETTIME
 
+#if HAVE_PTHREAD_CONDATTR_SETCLOCK
 bool
 fstrm__get_best_monotonic_clock_pthread(clockid_t *c)
 {
@@ -74,6 +75,7 @@ out:
 	assert(rc == 0);
 	return res;
 }
+#endif /* HAVE_PTHREAD_CONDATTR_SETCLOCK */
 
 bool
 fstrm__get_best_monotonic_clock_gettime(clockid_t *c)
@@ -116,6 +118,7 @@ fstrm__get_best_monotonic_clocks(clockid_t *clkid_gettime,
 		return false;
 	}
 
+#if HAVE_PTHREAD_CONDATTR_SETCLOCK
 	if (clkid_pthread != NULL &&
 	    !fstrm__get_best_monotonic_clock_pthread(clkid_pthread))
 	{
@@ -123,6 +126,7 @@ fstrm__get_best_monotonic_clocks(clockid_t *clkid_gettime,
 			*err = my_strdup("no clock available for pthread_cond_timedwait()");
 		return false;
 	}
+#endif
 
 	return true;
 }
